# Generated by Django 2.2.9 on 2019-12-21 19:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bom', '0024_auto_20191214_1342'),
    ]

    operations = [
        migrations.AlterField(
            model_name='partrevision',
            name='height_units',
            field=models.CharField(blank=True, choices=[('', '-----'), ('mil', 'mil'), ('in', 'in'), ('ft', 'ft'), ('yd', 'yd'), ('km', 'km'), ('m', 'm'), ('cm', 'cm'), ('um', 'μm'), ('nm', 'nm'), ('Other', 'Other')], default=None, max_length=5, null=True),
        ),
        migrations.AlterField(
            model_name='partrevision',
            name='length_units',
            field=models.CharField(blank=True, choices=[('', '-----'), ('mil', 'mil'), ('in', 'in'), ('ft', 'ft'), ('yd', 'yd'), ('km', 'km'), ('m', 'm'), ('cm', 'cm'), ('um', 'μm'), ('nm', 'nm'), ('Other', 'Other')], default=None, max_length=5, null=True),
        ),
        migrations.AlterField(
            model_name='partrevision',
            name='searchable_synopsis',
            field=models.TextField(blank=True, default='', editable=False, null=True),
        ),
        migrations.AlterField(
            model_name='partrevision',
            name='wavelength_units',
            field=models.CharField(blank=True, choices=[('', '-----'), ('km', 'km'), ('m', 'm'), ('cm', 'cm'), ('um', 'μm'), ('nm', 'nm'), ('A', 'Å'), ('Other', 'Other')], default=None, max_length=5, null=True),
        ),
        migrations.AlterField(
            model_name='partrevision',
            name='width_units',
            field=models.CharField(blank=True, choices=[('', '-----'), ('mil', 'mil'), ('in', 'in'), ('ft', 'ft'), ('yd', 'yd'), ('km', 'km'), ('m', 'm'), ('cm', 'cm'), ('um', 'μm'), ('nm', 'nm'), ('Other', 'Other')], default=None, max_length=5, null=True),
        ),
    ]
