# Generated by Django 2.1.3 on 2018-12-02 07:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bom', '0009_subpart_reference'),
    ]

    operations = [
        migrations.AddField(
            model_name='part',
            name='google_drive_parent',
            field=models.CharField(blank=True, default=None, max_length=128, null=True),
        ),
        migrations.AddField(
            model_name='usermeta',
            name='google_drive_parent',
            field=models.CharField(blank=True, default=None, max_length=128, null=True),
        ),
    ]
